@@ -1,8 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # check-code - a style and portability checker for Mercurial
 #
-# Copyright 2010 Matt Mackall <mpm@selenic.com>
+# Copyright 2010 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -620,13 +620,17 @@ cfilters = [
 ]
 
 inutilpats = [
-    [(r'\bui\.', "don't use ui in util"),],
+    [
+        (r'\bui\.', "don't use ui in util"),
+    ],
     # warnings
     [],
 ]
 
 inrevlogpats = [
-    [(r'\brepo\.', "don't use repo in revlog"),],
+    [
+        (r'\brepo\.', "don't use repo in revlog"),
+    ],
     # warnings
     [],
 ]
