@@ -1,6 +1,6 @@
 # destutil.py - Mercurial utility function for command destination
 #
-#  Copyright Matt Mackall <mpm@selenic.com> and other
+#  Copyright Olivia Mackall <olivia@selenic.com> and other
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -405,7 +405,7 @@ def desthistedit(ui, repo):
     elif default:
         revs = scmutil.revrange(repo, [default])
     else:
-        raise error.Abort(
+        raise error.ConfigError(
             _(b"config option histedit.defaultrev can't be empty")
         )
 
